
  (function() {
    const API_HOST = "https://api.keak.com";
    const test_id = 6;
    runTheSplitUrlTest();
    let test;
    let variant;
    let variant_order;
    let criteria_match = false;
    const start_time = new Date().getTime();
    const THIRTY_SECONDS_IN_MILLISECONDS = 30 * 1000;
    const ONE_MINUTE_IN_MILLISECONDS = 1 * 60 * 1000;
    let last_analytics_report_time = start_time;

    // Active time spent
    let active_time_spent = 0;

    // Rage click record credentials (requirements)
    const RAGE_CLICK_THREESHOLD = 5;
    const ALLOWED_RAGE_CLICK_INTERVAL = 5000; // milliseconds
    const RAGE_CLICK_TIMEFRAME = 999; // milliseconds
    const RAGE_CLICK_RADIUS = 30; // px
    let LAST_RAGE_CLICK_RECORD_TIME;
    let click_buffer = [];

    let maximum_scroll_depth;
    let removeLastSlashFromUrl;


    // Capture active time spent
    document.addEventListener("mousemove", () => {
      active_time_spent += 1;
    });

    document.addEventListener("keypress", () => {
      active_time_spent += 1;
    });

      // For rage clicks
    document.addEventListener("click", async (event) => {
      active_time_spent += 1;
      const click_time = new Date().getTime();
      click_buffer.push({ x: event.clientX, y: event.clientY, time: click_time });

      // Removing the old clicks
      click_buffer = click_buffer.filter((click) => click_time - click.time <= RAGE_CLICK_TIMEFRAME);

      // Checking for / recording rage clicks
      if(click_buffer.length >= RAGE_CLICK_THREESHOLD) {
        const first_click = click_buffer[0];
        const is_rage_click = click_buffer.every((click) => 
          Math.abs(click.x - first_click.x) <= RAGE_CLICK_RADIUS &&
          Math.abs(click.y - first_click.y) <= RAGE_CLICK_RADIUS);

        if(is_rage_click && criteria_match) {
          if (LAST_RAGE_CLICK_RECORD_TIME && click_time - LAST_RAGE_CLICK_RECORD_TIME <= ALLOWED_RAGE_CLICK_INTERVAL || !LAST_RAGE_CLICK_RECORD_TIME) {
            const analytics_data = {
              id: test_id,
              type: "rage_click",
              variant: variant_order,
              time_to_conversion: click_time - start_time,
              variation_id: test.variations[0]?.id,
              user_agent: window.navigator.userAgent,
              os: window.navigator?.platform,
              rage_clicks: click_buffer.length,
              sh: window.screen.height,
              sw: window.screen.width,
              language: window?.navigator?.userLanguage || window?.navigator?.language,
            };
            await recordAnalytics(analytics_data);
            LAST_RAGE_CLICK_RECORD_TIME = click_time;
            click_buffer = [];
          }
        }
      }
    });

    setInterval(async () => {
    const page_dwell_time = new Date().getTime() - last_analytics_report_time;
    if (criteria_match && removeLastSlashFromUrl(window.location.origin + window.location.pathname) == removeLastSlashFromUrl(variant)) {
      await reportTimeAndScrollAnalytics(page_dwell_time, maximum_scroll_depth);
      active_time_spent = 0;
      scroll_depth = 0;
      last_analytics_report_time = new Date().getTime();
    }
    localStorage.setItem("last_active_time", new Date().getTime());
  }, 2500);

    // Capture scroll depth change
    document.addEventListener("scroll", () => {
      const current_scroll_depth = window.scrollY;
      maximum_scroll_depth = Math.max(maximum_scroll_depth, current_scroll_depth);
    });

    removeLastSlashFromUrl = (url) => {
      if (url?.length > 0 && url[url.length - 1] === "/") {
        return url.substring(0, url.length - 1);
      }
      return url;
    };

    async function reportTimeAndScrollAnalytics(page_dwell_time, maximum_scroll_depth) {
      const url = API_HOST + "/split-url-tests/time-and-scroll-analytics";
      const requestOptions = {
        method: "POST",
        body: JSON.stringify({
          id: test_id,
          variant: variant_order,
          scroll_depth: maximum_scroll_depth,
          active_time_spent: active_time_spent,
          page_dwell_time: page_dwell_time
        }),
        headers: {
          "Content-Type": "application/json"
        }
      };

      try {
        const response = await fetch(url, requestOptions);
        const responseData = await response.json();

        if (responseData.data) {
          return responseData.data;
        } else {
          throw new Error("Response data format is not as expected");
        }
      } catch(err) {
        throw err;
      }
    }

    function getDeviceId() {
      const deviceId = localStorage.getItem("deviceId");
      if (deviceId) {
        return deviceId;
      } else {
        const min = 1;
        const max = 10000000;
        const deviceId = (Math.floor(Math.random() * (max - min + 1)) + min).toString();
        localStorage.setItem("deviceId", deviceId);
        return deviceId;
      }
    }

    function getSessionId() {
      const sessionId = localStorage.getItem("sessionId");
      if (sessionId) {
        return sessionId;
      } else {
        const newSessionId = (Math.floor(Math.random() * 1000000000)).toString();
        localStorage.setItem("sessionId", newSessionId);
        return newSessionId;
      }
    }

    async function getRandomSplitURLVariant(test_id) {
      try {
        let url = API_HOST + "/split-url-tests/" + test_id + "/variant?" + "device_id=" + getDeviceId();

        const variant_token = localStorage.getItem("variant_token");
        if (variant_token) {
          url += "&variant_token=" + variant_token;
        }

        const requestOptions = {
          method: "GET",
          headers: {
            "Content-Type": "application/json"
          }
        };

        const response = await fetch(url, requestOptions);
        const responseData = await response.json();

        if (responseData.data) {
          if (responseData.data.variant_token) {
            localStorage.setItem("variant_token", responseData.data.variant_token);
          }
          return responseData.data.variant;
        } else {
          throw new Error("Response data format is not as expected");
        }
      } catch (err) {
        throw new Error(err);
      }
    }

    async function runTheSplitUrlTest() {
      try {
        const test = await getTheSplitURLTest(test_id);
        if (test.criteria[0] && !checkTestCriteria(test.criteria[0])) {
          criteria_match = false;
          return;
        }
        criteria_match = true;

        // run the conversion page_view event
        await runConversionEvent(test.events[0].event, variant);
    
        if (removeLastSlashFromUrl(test.page_url) == removeLastSlashFromUrl(window.location.href)) {
            const query_parameters = window.location.search;
            location.replace(removeLastSlashFromUrl(variant) + (query_parameters ? query_parameters + "&" + "referrer=" + encodeURIComponent(test.page_url) : "?referrer=" + encodeURIComponent(test.page_url)));
        } else if (removeLastSlashFromUrl(window.location.origin + window.location.pathname) == removeLastSlashFromUrl(variant)) {
          const current_url = new URL(window.location.href);
          const search_params = new URLSearchParams(current_url.search);
          const referrer = decodeURIComponent(search_params.get("referrer"));

          const lastRequestTime = parseInt(localStorage.getItem("lastSplitUrlTestImpressionRequestTime" + test_id)) || 0;
          const currentTime = Date.now();
          if (removeLastSlashFromUrl(referrer) == removeLastSlashFromUrl(test.page_url) && currentTime - lastRequestTime >= THIRTY_SECONDS_IN_MILLISECONDS) { // Debounce for impressions
            localStorage.setItem("lastSplitUrlTestImpressionRequestTime" + test_id, currentTime.toString());

            const analytics_data = {
              id: test_id,
              type: "impression",
              variation_id: test.variations[0]?.id,
              time_to_conversion: new Date().getTime() - start_time,
              variant: variant_order,
              event_id: test.events[0]?.event?.id,
              device_id: getDeviceId(),
              session_id: getSessionId(),
              user_agent: window.navigator.userAgent,
              os: window.navigator?.platform,
              sh: window.screen.height,
              sw: window.screen.width,
              language: window?.navigator?.userLanguage || window?.navigator?.language,
              timezone: new Date().getTimezoneOffset() / -60,
              viewportWidth: window.innerWidth,
              viewportHeight: window.innerHeight,
              connection: window.navigator.connection?.effectiveType || 'unknown',
              referrer: document.referrer || 'direct',
              pageLoadTime: window.performance.timing.domContentLoadedEventEnd - window.performance.timing.navigationStart,
            };
            await recordAnalytics(analytics_data);
          }
        }
      } catch (err) {
        console.log(err);
      }
    }

    async function runConversionEvent(event, variant) {
      if (event && criteria_match) {
        if (
          removeLastSlashFromUrl(window.location.origin + window.location.pathname) !==
          removeLastSlashFromUrl(event.page_url)
        ) {
          localStorage.setItem("lastUrlSplitTestVariantSeen", variant);
        }

        let last_variant_seen = localStorage.getItem("lastUrlSplitTestVariantSeen");
        if (last_variant_seen) {
          let host_name = removeLastSlashFromUrl(window.location.origin + window.location.pathname);
          let page_url = removeLastSlashFromUrl(event.page_url);
          if (host_name.includes(page_url)) {
            await recordConversion(event.type, event.id, test.id, last_variant_seen);
          }
        }
      }
    }

    async function recordConversion(event_type, event_id, test_id, variant) {
      const lastRequestTime = parseInt(localStorage.getItem("lastSplitUrlTestConversionRequestTime" + event_id)) || 0;
      const currentTime = Date.now();
      if (currentTime - lastRequestTime >= ONE_MINUTE_IN_MILLISECONDS) { // Debounce for conversion events
        localStorage.setItem("lastSplitUrlTestConversionRequestTime" + event_id, currentTime.toString());

        const analytics_data = {
          id: test_id,
          type: "conversion",
          variation_id: test.variations[0]?.id,
          time_to_conversion: new Date().getTime() - start_time,
          variant: variant == test.variations[0]?.variant_a ? "variant_a" : "variant_b",
          event_id: event_id,
          device_id: getDeviceId(),
          session_id: getSessionId(),
          user_agent: window.navigator.userAgent,
          os: window.navigator?.platform,
          sh: window.screen.height,
          sw: window.screen.width,
          language: window?.navigator?.userLanguage || window?.navigator?.language,
          timezone: new Date().getTimezoneOffset() / -60,
          viewportWidth: window.innerWidth,
          viewportHeight: window.innerHeight,
          connection: window.navigator.connection?.effectiveType || 'unknown',
          referrer: document.referrer || 'direct',
          pageLoadTime: window.performance.timing.domContentLoadedEventEnd - window.performance.timing.navigationStart,
        };
        const response = await recordAnalytics(analytics_data);
      }
    }

    async function getTheSplitURLTest(test_id) {
      try {
        variant = await getRandomSplitURLVariant(test_id);
        const url = API_HOST + "/split-url-tests/" + test_id + "?variant=" + variant;
        const requestOptions = {
          method: "GET",
          headers: {
            "Content-Type": "application/json"
          }
        };

        const response = await fetch(url, requestOptions);
        const responseData = await response.json();
        if (responseData.data) {
          test = responseData.data;
          variant_order = variant == responseData.data.variations[0]?.variant_a ? "variant_a" : "variant_b";
          return responseData.data;
        } else {
          throw new Error("Response data format is not as expected");
        }
      } catch (error) {
        throw error;
      }
    }

    async function recordAnalytics(data) {
      const url = API_HOST + "/split-url-tests/analytics";
      const requestOptions = {
        method: "POST",
        body: JSON.stringify({...data}),
        headers: {
          "Content-Type": "application/json"
        }
      };

      try {
        const response = await fetch(url, requestOptions);

        const responseData = await response.json();

        if (responseData.data) {
          return responseData.data;
        } else {
          throw new Error("Response data format is not as expected");
        }
      } catch (error) {
        throw error;
      }
    }

    function checkTestCriteria(criteria) {
     // Check UTM Codes
      const urlParams = new URLSearchParams(window.location.search);
      const utmSource = urlParams.get("utm_source");
      const utmMedium = urlParams.get("utm_medium");
      const utmCampaign = urlParams.get("utm_campaign");
      const utmTerm = urlParams.get("utm_term");
      const utmContent = urlParams.get("utm_content");
      
      if(!criteria.devices.length && !criteria.browsers.length && !criteria.utm_source && !criteria.utm_medium && !criteria.utm_campaign && !criteria.utm_term && !criteria.utm_content && !criteria.languages.length && !criteria.regions.length) {
        return true;
      }

      // Check Device Type
      if (criteria.devices.length && !matchDeviceType(criteria.devices)) {
        return false;
      }
    
      // Check Browser Type
      if (criteria.browsers.length && !criteria.browsers.map(b => b.browser).includes(getBrowserType())) {
        return false;
      }

    
      if (utmSource && criteria.utm_source && criteria.utm_source !== utmSource) {
        return false;
      }
      if (utmMedium && criteria.utm_medium && criteria.utm_medium !== utmMedium) {
        return false;
      }
      if (utmCampaign && criteria.utm_campaign && criteria.utm_campaign !== utmCampaign) {
        return false;
      }
      if (utmTerm && criteria.utm_term && criteria.utm_term !== utmTerm) {
        return false;
      }
      if (utmContent && criteria.utm_content && criteria.utm_content !== utmContent) {
        return false;
      }
    
      // Check Language
      if (criteria.languages.length && !criteria.languages.map(l => l.language).includes(navigator.language)) {
        return false;
      }
    
      // Check Region (example using navigator.language for simplicity)
      if (criteria.regions.length && !criteria.regions.map(r => r.region).includes(navigator.language)) {
        return false;
      }
    
      return true;
    }
    
    function matchDeviceType(deviceTypes) {
      const ua = navigator.userAgent;
      return deviceTypes.some(deviceType => {
        if (deviceType.device === "mobile") {
          return /Mobi|Android|iPhone|iPod|Windows Phone/i.test(ua);
        } else if (deviceType.device === "tablet") {
          return /Tablet|iPad|Android(?!.*Mobile)/i.test(ua);
        } else if (deviceType.device === "desktop") {
          return !/Mobi|Android|Tablet|iPad|Windows Phone/i.test(ua);
        }
        return false;
      });
    }
    
    function getBrowserType() {
      const ua = navigator.userAgent;
      if (/chrome|crios|crmo/i.test(ua)) {
        return 'chrome';
      } else if (/firefox|fxios/i.test(ua)) {
        return 'firefox';
      } else if (/safari/i.test(ua)) {
        return 'safari';
      } else if (/opr|opera/i.test(ua)) {
        return 'opera';
      } else if (/edg/i.test(ua)) {
        return 'edge';
      } else {
        return 'other';
      }
    }

    // Start the test when the DOM is ready
    if (document.readyState !== "loading") {
      runTheSplitUrlTest();
    } else {
      document.addEventListener("DOMContentLoaded", runTheSplitUrlTest);
    }
  })();
